import { Injectable, EventEmitter } from '@angular/core';
import { Observable } from 'rxjs';
import { AngularFireAuth } from '@angular/fire/auth';
import { User, auth } from 'firebase';

@Injectable({
  providedIn: 'root'
})
export class AuthService {
  user$: Observable<User>;
  public updateUserData: EventEmitter<User>;
  public goToHomePage: EventEmitter<User>;
  public logoutEvent: EventEmitter<boolean>;

  constructor(public afAuth: AngularFireAuth) {
    this.updateUserData = new EventEmitter();
    this.goToHomePage = new EventEmitter();
    this.logoutEvent = new EventEmitter();
    this.getUserBasic();
  }

  getUserBasic() {
    this.afAuth.authState.subscribe(user => {
      if (user) {
        localStorage.setItem('user', JSON.stringify(user));
      } else {
        localStorage.setItem('user', null);
      }
    });
  }

  // Basic Login and Registration
  login(email: string, password: string) {
    try {
      this.afAuth.auth.signInWithEmailAndPassword(email, password).then(credential => {
        this.updateUserData.emit(credential.user);
        this.goToHomePage.emit(credential.user);
      });
    } catch (e) {
      alert('Error!' + e.message);
    }
  }
  registerUser(email: string, password: string) {
    return this.afAuth.auth.createUserWithEmailAndPassword(email, password).then(credential => {
      this.login(email, password);
    });
  }

  // Oauth Login and Registration
  oAuthLogin(provider) {
    return this.afAuth.auth.signInWithPopup(provider).then(credential => {
      this.updateUserData.emit(credential.user);
      this.goToHomePage.emit(credential.user);
    });
  }
  loginWithGoogle() {
    return this.oAuthLogin(new auth.GoogleAuthProvider());
  }
  loginWithTwitter() {
    return this.oAuthLogin(new auth.TwitterAuthProvider());
  }

  // logout
  logout() {
    this.logoutEvent.emit(true);
    this.afAuth.auth.signOut();
    localStorage.removeItem('user');
  }

  // helpers
  getUserFromStorage() {
    return JSON.parse(localStorage.getItem('user'));
  }

  public isLoggedIn(): boolean {
    const user = this.getUserFromStorage();
    return user !== null;
  }
}
