import { Component, OnInit, Input } from '@angular/core';
import { AngularFireStorage } from '@angular/fire/storage';
import { map } from 'rxjs/operators';
import { Observable } from 'rxjs';
import { UploadTaskSnapshot } from '@angular/fire/storage/interfaces';

@Component({
  selector: 'omnipresent-web-upload-file',
  templateUrl: './upload-file.component.html',
  styleUrls: ['./upload-file.component.scss']
})
export class UploadFileComponent implements OnInit {
  @Input() storagePath: string;
  @Input() accept: string;
  @Input() label: string;
  @Input() showProgress: boolean;
  uploadProgress: Observable<number>;
  image: string;
  id: string;

  constructor(private afStorage: AngularFireStorage) {
    this.accept = '.gif, .jpg, .png';
    this.label = 'Select Image One';
    this.id = this.newGuid();
  }

  ngOnInit() {}

  upload(event) {
    const ref = this.afStorage.ref(this.storagePath);
    const task = ref.put(event.target.files[0]);
    this.uploadProgress = task.snapshotChanges().pipe(map(s => (s.bytesTransferred / s.totalBytes) * 100));
    this.uploadProgress.subscribe(progress => {
      if (progress >= 100) {
        this.label = event.target.value.split('\\').pop() + '\nAdded';
      }
    });
  }

  newGuid() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, c => {
      const r = (Math.random() * 16) | 0;
      const v = c === 'x' ? r : (r & 0x3) | 0x8;
      return v.toString(16);
    });
  }
  // TODO: add this back to firebase rules once login works
  //service firebase.storage {
  //   match /b/{bucket}/o {
  //     match /{allPaths=**} {
  //       allow read, write: if request.auth != null;
  //     }
  //   }
  // }
}
