import { Component, OnInit } from '@angular/core';
import { AngularFirestoreDocument, AngularFirestore } from '@angular/fire/firestore';
import { User } from 'firebase';
import { Profile } from '../models/profile';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { Router } from '@angular/router';

@Component({
  selector: 'app-login-tester',
  templateUrl: './login-tester.component.html',
  styleUrls: ['./login-tester.component.scss']
})
export class LoginTesterComponent implements OnInit {
  constructor(private afs: AngularFirestore, private router: Router) {}

  ngOnInit() {}

  loginWithGoogle() {
    console.log('I have logged in with google');
  }

  updateUserData(user: User) {
    const userRef: AngularFirestoreDocument<any> = this.afs.doc(`profile/${user.uid}`);
    const data: Profile = {
      name: user.displayName,
      email: user.email
    };
    return userRef.set(data, { merge: true });
  }
  updateAdminData(user: User) {
    const userRef: AngularFirestoreDocument<any> = this.afs.doc(`profile/${user.uid}`);
    const data: Profile = {
      name: user.displayName,
      email: user.email,
      roles: ['admin']
    };
    return userRef.set(data, { merge: true });
  }
  updateViewData(user: User) {
    const userRef: AngularFirestoreDocument<any> = this.afs.doc(`profile/${user.uid}`);
    const data: Profile = {
      name: user.displayName,
      email: user.email,
      roles: ['view']
    };
    return userRef.set(data, { merge: true });
  }

  getUserRoles(uid: string): Observable<string[]> {
    return this.afs
      .doc<Profile>(`profile/` + uid)
      .valueChanges()
      .pipe(
        map(user => {
          if (user) {
            return user.roles;
          }
        })
      );
  }

  goToHomePage(user: User) {
    this.getUserRoles(user.uid).subscribe(roles => {
      if (roles) {
        if (roles.includes('admin')) {
          this.router.navigate(['upload']);
        } else if (roles.includes('view')) {
          this.router.navigate(['notification']);
        } else {
          this.router.navigate(['profile']);
        }
      } else {
        this.router.navigate(['profile']);
      }
    });
  }
  logout(event) {
    // this is needed until we add notifications
    console.log('logging out');
    this.router.navigate(['login']);
  }
}
